/*
 * Author: Harry van Haaren 2013
 *         harryhaaren@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#ifndef AVTK_H
#define AVTK_H

#include "avtk/avtk_helpers.h"

#include "avtk/avtk_lfo.h"
#include "avtk/avtk_pad.h"
#include "avtk/avtk_adsr.h"
#include "avtk/avtk_dial.h"
#include "avtk/avtk_image.h"
#include "avtk/avtk_opengl.h"
#include "avtk/avtk_reverb.h"
#include "avtk/avtk_button.h"
#include "avtk/avtk_volume.h"
#include "avtk/avtk_waveform.h"
#include "avtk/avtk_compressor.h"
#include "avtk/avtk_oscillator.h"
#include "avtk/avtk_background.h"
#include "avtk/avtk_filtergraph.h"
#include "avtk/avtk_light_button.h"

#endif // AVTK_H
