/*
 * Author: Harry van Haaren 2013
 *         harryhaaren@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#ifndef FABLA_SAMPLE_H
#define FABLA_SAMPLE_H

#include "shared.h"

class Sample
{
  public:
    Sample()
    {
      data = 0;
      path = 0;
      path_len = 0;
      
      speed = 1.0f;
    }
    
    SF_INFO info;      // Info about sample from sndfile
    float*  data;      // Sample data in float
    char*   path;      // Path of file
    size_t  path_len;  // Length of path
    
    float gain;
    float speed;
    float pan;
};

#endif // FABLA_SAMPLE_H
